import os, sys
import _gtk
from gtk import *
from GDK import *
from string import *
from popen2 import *

import Error
from string import split														# String Functions
from Constants import TRUE, FALSE												# Boolean Values
from Constants import PI_NAME, PI_LOCALE_NAME, PI_VERSION, PI_ABOUT				# Program Information
from Constants import PI_COMMANDS_DIR, PI_LOCALE_DIR, PI_ICONS_DIR				# Program Directories
from Constants import ST_LOCAL_IS_NEWER, ST_REMOTE_IS_NEWER, ST_EITHER_IS_NEWER	# Synchronization types
from Constants import CT_RSH, CT_SSH, CT_FTP									# Protocols
from Constants import ST_STRINGS, CT_STRINGS									# String Representations
from Constants import URL_MANDRAKE_CAMPUS, URL_MANDRAKE_EXPERT, URL_MANDRAKE_QA	# URLs
from Constants import *															# Finally import everything (Menus)
from FileTree import FTPFileTree, LocalFileTree									# File Trees
from SyncFileTree import SyncFileTree, SyncFile									# Synchronized File Tree
from Connection import FTPConnection, LocalConnection							# Connection Types

# ------------------------------------------------------------------------------
# Localization

import gettext
gettext.bindtextdomain(PI_LOCALE_NAME, PI_LOCALE_DIR)
gettext.textdomain(PI_LOCALE_NAME)
_ = gettext.gettext

# ------------------------------------------------------------------------------
# Backwards compatibility
# (used only when reading an old preferences file)

NONE_IS_NEWER	= ST_EITHER_IS_NEWER
LOCAL_IS_NEWER	= ST_LOCAL_IS_NEWER
REMOTE_IS_NEWER	= ST_REMOTE_IS_NEWER

PROTOCOL_SSH	= CT_SSH
PROTOCOL_RSH	= CT_RSH
PROTOCOL_FTP	= CT_FTP

# ------------------------------------------------------------------------------
# Program dependencies

def check_program(progname):
	exists = FALSE
	if os.environ.has_key("PATH"):
		for dir in split(os.environ["PATH"], ":"):
			exename = os.path.join(dir, progname)
			if os.access(exename, os.X_OK):
				exists = TRUE
	return exists

have_rsync		= check_program("rsync")
have_ssh		= check_program("ssh")
have_rsh		= check_program("rsh")

# Some static configuration
# Programs (Specified as (command, (arg1, arg2, ...)))
# Program to launch to show some help
if os.environ.has_key("HELP_BROWSER"):
	command_help = ("/bin/sh", ("-c", os.environ["HELP_BROWSER"] + " " + "file:///usr/share/doc/DrakSync/help.html",))
else:
	command_help = (None, ())

# Misc constants
BORDER_WIDTH = 5
GRAB_DEFAULT = TRUE

# This is a function that just starts another program, without caring
# anything about it after the spawn...
def spawn(command, args):
	if command:
		pid = os.fork()
		if pid == 0:
			os.execvp(command, (command, ) + args)
		else:
			return pid

# General help viewer. The args are a path, e.g. show_help("foo",
# "bar", "gazonk").
def show_help(*args):
	apply(spawn, command_help)

def save():
	fd = open(save_path, "w")
	for item in save_objects:
		fd.write(item() + "\n")
	fd.close()

def load():
	# Try to load config-file
	if os.path.exists(save_path):
		execfile(save_path)

def save_lines():
	return repr(gui)

save_path = os.path.expanduser("~/.draksync")
save_objects = [save_lines]

# Unqueue pending GtkEvents
def gtk_iteration():
	while events_pending():
		mainiteration()

# ------------------------------------------------------------------------------
# Sensitivity handling

sensitive_list_sync_all					= []	# group of widgets for synchronizing everything
sensitive_list_sync_group				= []	# group of widgets for group synchronization
sensitive_list_sync_connection			= []	# group of widgets for connection synchronization
sensitive_list_group_if_active			= []	# group of widgets if a group is active
sensitive_list_group_if_inactive		= []	# group of widgets if no group is active
sensitive_list_connection_if_active		= []	# group of widgets if a connection is selected
sensitive_list_connection_if_inactive	= []	# group of widgets if no connection is selected

def strip_underscore(in_string):
	out_string = ""
	for sub_string in split(in_string, "_"):
		out_string += sub_string
	return out_string

def set_sensitive(items, active):
	for item in items:
		item.set_sensitive(active)

def update_sensitive_lists(profiles):
	profile = profiles.current()
	if not profile:
		# No active profile, it's only possible to create a new group
		set_sensitive(sensitive_list_sync_all, FALSE)
		set_sensitive(sensitive_list_group_if_inactive, TRUE)
		set_sensitive(sensitive_list_group_if_active, FALSE)
		set_sensitive(sensitive_list_connection_if_inactive, FALSE)
		set_sensitive(sensitive_list_connection_if_active, FALSE)
	else:
		# Defaults to "inactive mode" except for group
		set_sensitive(sensitive_list_group_if_inactive, TRUE)
		set_sensitive(sensitive_list_connection_if_inactive, TRUE)
		set_sensitive(sensitive_list_group_if_active, TRUE)
		set_sensitive(sensitive_list_connection_if_active, FALSE)

		# Enable all synchronization if there is actually
		# something to synchronize in one of the groups
		exists_connection = FALSE
		for profile in profiles.profiles:
			exists_connection = exists_connection or (len(profile.items) != 0)
		set_sensitive(sensitive_list_sync_all, exists_connection)

		# Enable group synchronization if there is actually
		# something to synchronize in the active group
		profile = profiles.current()
		exists_connection = len(profile.items) != 0
		set_sensitive(sensitive_list_sync_group, exists_connection)

		item = profile.current()
		if not item:
			# No active item, offers the possibility only for creating a new connection
			set_sensitive(sensitive_list_connection_if_active, FALSE)
		else:
			set_sensitive(sensitive_list_connection_if_inactive, TRUE)
			set_sensitive(sensitive_list_connection_if_active, TRUE)
	

# ------------------------------------------------------------------------------
# GUI wrapper over Gtk

class GuiPopup:
	class ui_btn_pressed:
		def __init__(self, dialog, user):
			self.dialog = dialog
			self.userfn = user[0]
			self.userdata = user[1]
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, self.userdata)
				else:
					apply(self.userfn, ())

class GuiPopupQuestion(GtkDialog, GuiPopup):
	def __init__(self, title, question, cancel = (None, None), buttons = ()):
		GuiPopupQuestion.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.set_position(WIN_POS_CENTER)
		self.set_title(title)
		self.connect("destroy", self.ui_btn_pressed(self, cancel))
		self.innervbox = GtkVBox(10)
		self.innervbox.set_border_width(10)
		self.question = GtkLabel(question)
		self.innervbox.pack_start(self.question, TRUE, TRUE, 0)
		self.vbox.add(self.innervbox)
		self.buttons = []
		self.default_button = None
		for button_info in buttons:
			# Tuple format: ( label , callback [, arg = None [, grab_default = FALSE ] ] )
			(label, callback), arg, grab_default  = button_info[0:2], None, FALSE
			if len(button_info) >= 3: arg = button_info[2]
			if len(button_info) >= 4: grab_default = button_info[3]
			button = GtkButton(label)
			self.action_area.pack_start(button)
			self.buttons.append(button)
			button.connect("clicked", self.ui_btn_pressed(self, (callback, arg)))
			# FIXME: set CAN_DEFAULT flag for *every* button since
			# non-"grab_default"'ed buttons will appear much bigger...
			button.set_flags(CAN_DEFAULT)
			if grab_default:
				button.grab_default()
				self.default_button = button
	def show(self):
		self.innervbox.show()
		self.question.show()
		for button in self.buttons:
			button.show()
		GuiPopupQuestion.__bases__[0].show(self)

class GuiPopupInput(GuiPopupQuestion):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupInput.__bases__[0].__init__(self, title, question, cancel, buttons)
		self.input = GtkEntry()
		self.input.set_text(orig)
		self.input.connect("activate", self.ui_enter_pressed)
		self.innervbox.pack_start(self.input, TRUE, TRUE, 0)
	def show(self):
		self.input.show()
		self.input.grab_focus()
		GuiPopupInput.__bases__[0].show(self)
	def ui_enter_pressed(self, *args):
		if self.default_button:
			self.default_button.clicked()
	class ui_btn_pressed(GuiPopup.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.input.get_text(),) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.input.get_text(),))

class GuiPopupPasswordInput(GuiPopupInput):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupPasswordInput.__bases__[0].__init__(self, title, question, orig, cancel, buttons)
		self.input.set_visibility(FALSE)

class GuiPopupNewPasswordInput(GuiPopupPasswordInput):
	def __init__(self, title, question, orig, cancel = (None, None), buttons = ()):
		GuiPopupNewPasswordInput.__bases__[0].__init__(self, title, question, orig, cancel, buttons)
		self.input2 = GtkEntry()
		self.input2.set_text(orig)
		self.input2.set_visibility(FALSE)
		self.innervbox.pack_start(self.input2, TRUE, TRUE, 0)
	def show(self):
		self.input2.show()
		GuiPopupNewPasswordInput.__bases__[0].show(self)
	class ui_btn_pressed(GuiPopupPasswordInput.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.input.get_text(), self.dialog.input.get_text() == self.dialog.input2.get_text()) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.input.get_text(), self.dialog.input.get_text() == self.dialog.input2.get_text()))

class GuiPopupNewPasswordInputOKCancel(GuiPopupNewPasswordInput):
	def __init__(self, title, question, orig, ok = (None, None), cancel = (None, None)):
		GuiPopupNewPasswordInputOKCancel.__bases__[0].__init__(self, title, question, orig, (self.ui_cancel, cancel[1]), ((_("OK"), self.ui_ok, ok[1]), (_("Cancel"), self.ui_cancel, cancel[1])))
		self.ok = ok[0]
		self.cancel = cancel[0]
	def ui_cancel(self, passwd, equal, *args):
		apply(self.cancel, args)
	def ui_ok(self, passwd, equal, *args):
		if equal:
			apply(self.ok, (passwd,) + args)
		else:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Passwords do not match"),
							 _("The two passwords you entered do not match.") + "\n" +
							 _("Please retry."),
							 (self.cancel, None), ((_("OK"), self.ui_retry, None, GRAB_DEFAULT),)).show()
	def ui_retry(self):
		self.show()

# This is just an ordinary file selection box, but with only directory
# selection widgets visible...
class GuiPopupDirectorySelection(GtkFileSelection, GuiPopup):
	def __init__(self, title, orig=None, cancel=(None, None), ok=(None, None)):
		GuiPopupDirectorySelection.__bases__[0].__init__(self, title)
		self.hide_fileop_buttons()
		scroller = _gtk.gtk_object_get(self.file_list._o, "parent")
		_gtk.gtk_widget_hide(scroller)
		if orig:
			self.set_filename(orig)
		self.connect("destroy", self.ui_btn_pressed(self, cancel))
		self.cancel_button.connect("clicked", self.ui_btn_pressed(self, cancel))
		self.ok_button.connect("clicked", self.ui_btn_pressed(self, ok))
	class ui_btn_pressed(GuiPopup.ui_btn_pressed):
		def __call__(self, button):
			self.dialog.hide()
			if self.userfn:
				if self.userdata != None:
					apply(self.userfn, (self.dialog.get_filename(),) + self.userdata)
				else:
					apply(self.userfn, (self.dialog.get_filename(),))

class Identity:
	def __got_input__(self, *args):
		input = self.adder.read(1)
		if input == "":
			self.cancel()
		elif input == "\n":
			# These are the input categories:
			#ERROR[: .*]
			if find(self.input, "ERROR") == 0:
				if self.input == "ERROR: PERMISSION DENIED":
					if not self.adder.closed:
						_gtk.gtk_input_remove(self.input_tag)
						self.adder.close()
					GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Permission denied"),
										  _("Permission denied. Please reenter the passphrase for your ssh identity:"),
										  "", (self.ui_password_cancel, None),
										  ((_("OK"), self.ui_password_ok, None, GRAB_DEFAULT), (_("Cancel"), self.ui_password_cancel, None))).show()
				elif self.input == "ERROR: NO SSH-AGENT":
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Installation problem"),
									 _("You seem not to have ssh-agent installed.") + " " +
									 _("Please verify that you have ssh installed correctly and your PATH set up to include the installation directory of ssh."),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.cancel()
				else:
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Unknown error"),
									 _("An unknown error '%s' has occurred.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.cancel()
			elif self.input == "DONE": 
				self.cont()
			self.input = ""
		else:
			self.input = self.input + input
	def __init__(self):
		self.continues = []
	def add(self, cont = (None, None), cancel = (None, None)):
		first = self.continues == []
		self.continues.append((cont, cancel))
		if first:
			GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Passphrase required for ssh identity"),
								  _("Your ssh identity requires a passphrase.") + "\n" +
								  _("Please enter your passphrase:"),
								  "", (self.ui_password_cancel, None),
								  ((_("OK"), self.ui_password_ok, None, GRAB_DEFAULT), (_("Cancel"), self.ui_password_cancel, None))).show()
	def ui_password_ok(self, passwd):
		self.input = ""
		self.adder, self.adderin = popen2(PI_COMMANDS_DIR + "/unsetlang " + PI_COMMANDS_DIR + "/add_identity.exp 2>&1", 0)
		self.adderin.write("PASSWORD: " + passwd + "\n")
		self.input_tag = _gtk.gtk_input_add(self.adder.fileno(), INPUT_READ, self.__got_input__);
	def ui_password_cancel(self, text):
		self.cont_cancel()
	def cont(self):
		if not self.adder.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.adder.close()
		for cont in self.continues:
			if cont[0][1]:
				apply(cont[0][0], cont[0][1])
			else:
				apply(cont[0][0], ())
		self.continues = []
	def cont_cancel(self):
		for cont in self.continues:
			if cont[1][1]:
				apply(cont[1][0], cont[1][1])
			else:
				apply(cont[1][0], ())
		self.continues = []
	def cancel(self):
		if not self.adder.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.adder.close()
		self.cont_cancel()
identity = Identity()

# ------------------------------------------------------------------------------
# Transfer (FTP protocol)

class FTPTransfer:

	def __init__(self, sync_type, delete_files, hostname, username, remote_root_dir, local_root_dir):
		self.protocol			= CT_FTP
		self.sync_type			= sync_type
		self.delete_files		= delete_files
		self.hostname			= hostname
		self.username			= username
		self.remote_root_dir	= remote_root_dir
		self.local_root_dir		= local_root_dir
		self.first_auth_attempt	= TRUE
		self.cancel				= FALSE
		self.local_connection	= None
		self.remote_connection	= None
		self.sync_file_tree		= None

		# UI initialization
		self.main_window = GtkWindow()
		self.main_window.connect("delete_event", self.cancel_cb)
		self.main_window.set_default_size(400, 150)
		self.main_window.set_title(_("Synchronization in progress"))
		self.main_window.set_border_width(10)

		self.main_vbox = GtkVBox()
		self.label_remaining_files = GtkLabel(_("Please wait while determining the number of files to synchronize"))
		self.main_vbox.pack_start(self.label_remaining_files, TRUE, FALSE, 4)
		self.bar_all_files = GtkProgressBar()
		self.main_vbox.pack_start(self.bar_all_files, TRUE, FALSE, 4)
		self.label_current_file = GtkLabel(_("Current file: %s") % "")
		self.main_vbox.pack_start(self.label_current_file, TRUE, FALSE, 4)
		self.bar_current_file = GtkProgressBar()
		self.main_vbox.pack_start(self.bar_current_file, TRUE, FALSE, 4)
		button = GtkButton(_("Cancel"))
		button.connect("clicked", self.cancel_cb)
		self.main_vbox.pack_start(button, FALSE, FALSE, 4)

		self.main_window.add(self.main_vbox)
		self.main_window.show_all()

		self.__initiate_connection()

	def __initiate_connection(self):
		if self.first_auth_attempt:
			self.first_auth_attempt = FALSE
			GuiPopupPasswordInput(_("%s - Password required for remote host") % PI_NAME,
								  _("The remote host requires a password for %s@%s.") % (self.username, self.hostname) + "\n" +
								  _("Please enter your password:"),
								  "", (self.__password_cancel_cb, None),
								  ((_("OK"), self.__password_ok_cb, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel_cb, None))).show()
		else:
			GuiPopupPasswordInput(_("%s - Permission denied") % PI_NAME,
								  _("Permission denied.") + " " +
								  _("Please enter your password again for %s@%s:") % (self.username, self.hostname),
								  "", (self.__password_cancel_cb, None),
								  ((_("OK"), self.__password_ok_cb, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel_cb, None))).show()

	def __password_ok_cb(self, text):
		self.password = text
		try:
			self.__initiate_file_tree()
		except Error.LoginFailed:
			self.__initiate_connection()
		
	def __password_cancel_cb(self, text):
		self.__delete_self()

	def __delete_self(self):
		self.bar_current_file.destroy()
		self.bar_all_files.destroy()
		self.label_remaining_files.destroy()
		self.main_vbox.destroy()
		self.main_window.destroy()
		del self

	def __initiate_file_tree(self):
		try:
			remote_connection = FTPConnection(self.hostname, self.username, self.password)
			remote_file_tree = FTPFileTree(remote_connection)
			remote_file_tree.getTree(self.remote_root_dir)
		except Error.ConnectionClosed:
			self.cancel = TRUE
			GuiPopupQuestion(_("%s - Connection closed") % PI_NAME,
							 _("Service not available, remote server closed the connection"),
							 (None, None), ((_("OK"), self.cancel_cb, None, GRAB_DEFAULT),)).show()
		except Error.DirectoryNotFound, dirname:
			self.cancel = TRUE
			# TODO: popup dialog to ask for the creation of the remote directory/directories
			action = _("Please create the %s directory before attempting to synchronize your files.") % _("remote")
			GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
							 _("The specificed %s directory (%s) does not exist.") % (_("remote"), dirname) + "\n" + action,
							 (None, None), ((_("OK"), self.cancel_cb, None, GRAB_DEFAULT),)).show()

		try:
			if not self.cancel:
				local_connection = LocalConnection()
				local_file_tree = LocalFileTree(local_connection)
				local_file_tree.getTree(self.local_root_dir)
		except Error.DirectoryNotFound, dirname:
			self.cancel = TRUE
			# TODO: popup dialog to ask for the creation of the local directory/directories
			action = _("Please create the %s directory before attempting to synchronize your files.") % _("local")
			GuiPopupQuestion(_("%s - Directory not found") % PI_NAME,
							 _("The specificed %s directory (%s) does not exist.") % (_("local"), dirname) + "\n" + action,
							 (None, None), ((_("OK"), self.cancel_cb, None, GRAB_DEFAULT),)).show()

		try:
			if not self.cancel:
				self.sync_file_tree = SyncFileTree(self.sync_type, local_file_tree, remote_file_tree)
				self.sync_file_tree.prepare()
				self.__synchronize()
		except Error.ConnectionClosed:
			self.cancel = TRUE
			GuiPopupQuestion(_("%s - Connection closed") % PI_NAME,
							 _("Service not available, remote server closed the connection"),
							 (None, None), ((_("OK"), self.cancel_cb, None, GRAB_DEFAULT),)).show()

	def __gui_update_dir_entries(self, local_dir, remote_dir, dirname):
		# TODO: show current local and remote directories
		gtk_iteration()
		
	def __gui_update_file_entry(self, filename):
		self.label_current_file.set_text(_("Current file: %s") % filename)
		gtk_iteration()
		return self.cancel

	def __gui_update_file_progress(self, percent):
		self.bar_current_file.update(percent)
		gtk_iteration()

	def __gui_update_files_progress(self):
		self.file_count = self.file_count + 1
		self.bar_current_file.set_value(0)
		self.bar_all_files.update(float(self.file_count) / float(self.total_file_count))
		self.label_remaining_files.set_text(_("Files remaining to process: %d") % (self.total_file_count - self.file_count))
		gtk_iteration()

	def __count_files_cb(self, path, sync_file, __unused_arg):
		if sync_file.isfile():
			self.total_file_count = self.total_file_count + 1

	def __synchronize(self):
		self.file_count = 0
		self.total_file_count = 0
		self.sync_file_tree.traverse(self.__count_files_cb, None)
		self.label_remaining_files.set_text(_("Files remaining to process: %d") % self.total_file_count)

		self.bar_all_files.set_bar_style(PROGRESS_CONTINUOUS)
		self.bar_all_files.set_orientation(PROGRESS_LEFT_TO_RIGHT)
		self.bar_all_files.set_value(0)
		
		self.bar_current_file.set_bar_style(PROGRESS_CONTINUOUS)
		self.bar_current_file.set_orientation(PROGRESS_LEFT_TO_RIGHT)
		self.bar_current_file.set_value(0)
		
		self.sync_file_tree.synchronize((
			self.__gui_update_dir_entries,
			self.__gui_update_file_entry,
			self.__gui_update_file_progress,
			self.__gui_update_files_progress,
			self.delete_files
			))
		self.__delete_self()
	
	def cancel_cb(self, *args):
		self.cancel = TRUE
		self.__delete_self()

# ------------------------------------------------------------------------------
# Transfer (rsync protocol)

class Transfer:
	def __got_input__(self, *args):
		input = self.syncer.read(1)
		if input == "":
			self.__delete__()
		elif input == "\n":
			# These are the input categories:
			#ERROR[: .*]
			if find(self.input, "ERROR") == 0:
				_gtk.gtk_input_remove(self.input_tag)
				self.syncer.close()
				if self.input == "ERROR: PERMISSION DENIED":
					if self.first_time:
						GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Passphrase required for host"),
											  _("The remote host requires a passphrase for %s@%s.") % (self.user, self.host) + "\n" +
											  _("Please enter your passphrase:"),
											  "", (self.__password_cancel__, None),
											  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
					else:
						GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Permission denied"),
											  _("Permission denied. Please reenter passphrase for %s@%s:") % (self.user, self.host),
											  "", (self.__password_cancel__, None),
											  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
					self.first_time = FALSE
				elif self.input == "ERROR: NO SSH-AGENT":
					identity.add((self.__initiate_sync__, None), ( self.__delete__, None))
				elif self.input == "ERROR: CONNECTION CLOSED":
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Remote host closed the connection"),
									 _("The remote host closed the connection for %s@%s.") % (self.user, self.host) + "\n" +
									 _("This is probably due to that either the destination or the source directory (or both) does not exist.") + "\n" +
									 _("Please create the directory/ies before attempting to synchronize them."),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
				else:
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Unknown error"),
									 _("An unknown error '%s' has occurred.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
			#NUMBER OF FILES TO TRANSFER [IN (ONE|THE OTHER DIRECTION)]
			elif find(self.input, "NUMBER OF FILES TO TRANSFER") == 0:
				if find(self.input, "NUMBER OF FILES TO TRANSFER:") == 0 or find(self.input, "NUMBER OF FILES TO TRANSFER IN ONE DIRECTION:") == 0:
					self.number_of_files_to_transfer_in_one_direction = int(self.input[find(self.input, ":") + 1:])
				else:
					self.number_of_files_to_transfer_in_the_other_direction = int(self.input[find(self.input, ":") + 1:])
			#TRANSFERING [IN (ONE|THE OTHER) DIRECTION]
			elif find(self.input, "TRANSFERING") == 0:
				if self.input == "TRANSFERING" or self.input == "TRANSFERING IN ONE DIRECTION":
					self.eachfile = 1
					self.all.configure(0, 0, self.number_of_files_to_transfer_in_one_direction);
					if self.number_of_files_to_transfer_in_one_direction > 1:
						self.all.set_bar_style(PROGRESS_DISCRETE)
						self.all.set_discrete_blocks(self.number_of_files_to_transfer_in_one_direction)
					else:
						self.all.set_bar_style(PROGRESS_CONTINUOUS)
					self.all.set_orientation(PROGRESS_LEFT_TO_RIGHT)
				elif self.input == "TRANSFERING IN THE OTHER DIRECTION":
					self.eachfile = -1
					self.all.configure(self.number_of_files_to_transfer_in_the_other_direction, 0, self.number_of_files_to_transfer_in_the_other_direction);
					if self.number_of_files_to_transfer_in_the_other_direction > 1:
						self.all.set_bar_style(PROGRESS_DISCRETE)
						self.all.set_discrete_blocks(self.number_of_files_to_transfer_in_the_other_direction)
					else:
						self.all.set_bar_style(PROGRESS_CONTINUOUS)
					self.all.set_orientation(PROGRESS_RIGHT_TO_LEFT)
			#TRANSFERED (BYTES|PERCENT): [0123456789]*
			elif find(self.input, "TRANSFERED") == 0:
				if find(self.input, "TRANSFERED PERCENT") == 0:
					self.current.set_value(int(self.input[find(self.input, ":") + 1:]))
			#TRANSFER [IN (ONE|THE OTHER) DIRECTION] DONE
			elif find(self.input, "TRANSFER ") == 0:
				if find(self.input, "TRANSFER DONE") == 0 or find(self.input, "TRANSFER IN THE OTHER DIRECTION DONE") == 0:
					self.__delete__()
					return
			#ONE MORE FILE TRANSFERED
			elif self.input == "ONE MORE FILE TRANSFERED":
				self.all.set_value(self.all.get_value() + self.eachfile)
				self.current.set_value(0)
			self.input = ""
		else:
			self.input = self.input + input
	def __initiate_sync__(self):
		if self.protocol == CT_SSH or self.protocol == CT_RSH:
			local = os.path.expanduser(self.local)
			if local[-1:] != '/':
				local = local + "/"
			if self.host != "localhost" or self.user != os.environ["USER"]:
				remote = self.user + "@" + self.host + _(":") + self.remote
			else:
				remote = os.path.expanduser(self.remote)
			if remote[-1:] != '/':
				remote = remote + "/"
			if self.protocol == CT_SSH:
				os.environ["RSYNC_RSH"] = 'ssh'
			else:
				os.environ["RSYNC_RSH"] = 'rsh'
			if self.delete_files:
				os.environ["DELETE_FILES"] = 'yes'
			else:
				os.environ["DELETE_FILES"] = 'no'
			if self.newer == ST_EITHER_IS_NEWER:
				command = PI_COMMANDS_DIR + "/none_is_newer.exp"
			else:
				command = PI_COMMANDS_DIR + "/some_is_newer.exp"
			if self.newer == ST_LOCAL_IS_NEWER:
				dirs = (local, remote)
			else:
				dirs = (remote, local)
			self.syncer, self.syncerin = popen2(PI_COMMANDS_DIR + "/unsetlang " + command + " \"" + dirs[0] + "\" \"" + dirs[1] + "\" 2>&1", 0)
			self.syncerin.write("PASSWORD: " + self.passwd + "\n")
			self.input_tag = _gtk.gtk_input_add(self.syncer.fileno(), INPUT_READ, self.__got_input__);
		elif self.protocol == CT_FTP:
			pass
		else:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Bug: ") + _("Unsupported protocol"),
							 _("The protocol specified is not supported.") + " " + _("This is a bug - please report this."),
							 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
			self.__delete__()
	def __password_ok__(self, text):
		self.passwd = text
		self.__initiate_sync__()  
	def __password_cancel__(self, text):
		self.__delete__()
	def __init__(self, protocol=CT_SSH, local="~/tmp", user="guest", host="localhost", remote="~/tmp", newer=ST_EITHER_IS_NEWER, delete_files=TRUE):
		self.protocol = protocol
		self.local = local
		self.user = user
		self.host = host
		self.remote = remote
		self.newer = newer
		self.delete_files = delete_files
		self.passwd = ""
		self.first_time = 1
		self.input = ""
		self.main = GtkWindow()
		self.main.connect("delete_event", self.ui_delete)
		self.main.set_default_size(400, 150)
		self.main.set_title(_("Synchronization in progress"))
		self.main.set_border_width(10)
		self.container = GtkVBox()
		if newer == ST_REMOTE_IS_NEWER:
			self.title = GtkLabel(_("Synchronizing %s with %s@%s:%s.") % (self.local, self.user, self.host, self.remote) + " " + _("Please wait."))
		elif newer == ST_LOCAL_IS_NEWER:
			self.title = GtkLabel(_("Synchronizing %s@%s:%s with %s.") % (self.user, self.host, self.remote, self.local) + " " + _("Please wait."))
		else:
			self.title = GtkLabel(_("Synchronizing %s and %s@%s:%s.") % (self.local, self.user, self.host, self.remote) + " " + _("Please wait."))
		self.container.pack_start(self.title, 1, 0, 0)
		self.all_title = GtkLabel(_("Files:"))
		self.container.pack_start(self.all_title, 1, 0, 0)
		self.all = GtkProgressBar()
		self.all.set_bar_style(PROGRESS_DISCRETE)
		self.container.pack_start(self.all, 1, 0, 0)
		self.current_title = GtkLabel(_("Current file:"))
		self.container.pack_start(self.current_title, 1, 0, 0)
		self.current = GtkProgressBar()
		self.current.set_bar_style(PROGRESS_CONTINUOUS)
		self.current.set_orientation(PROGRESS_LEFT_TO_RIGHT)
		self.container.pack_start(self.current, 1, 0, 0)
		self.cancel = GtkButton(_("Cancel"))
		self.cancel.connect("clicked", self.ui_delete)
		self.container.pack_start(self.cancel, 1, 0, 0)
		self.main.add(self.container)
		self.main.show_all()
		self.__initiate_sync__()
	def __delete__(self):
		if not self.syncer.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.syncer.close()
		self.current.destroy()
		self.current_title.destroy()
		self.all.destroy()
		self.all_title.destroy()
		self.title.destroy()
		self.container.destroy()
		self.main.destroy()
		del self
	def ui_delete(self, *args):
		self.__delete__()
		return TRUE

class TransferIdentity:
	def __got_input__(self, *args):
		input = self.transfer.read(1)
		if input == "":
			self.__delete__()
		elif input == "\n":
			# These are the input categories:
			#ERROR[: .*]
			if find(self.input, "ERROR") == 0:
				_gtk.gtk_input_remove(self.input_tag)
				self.transfer.close()
				if self.input == "ERROR: PERMISSION DENIED":
					GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Permission denied"),
										  _("Permission denied. Please reenter passphrase for %s@%s:") % (self.user, self.host),
										  "", (self.__password_cancel__, None),
										  ((_("OK"), self.__password_ok__, None, GRAB_DEFAULT), (_("Cancel"), self.__password_cancel__, None))).show()
				elif self.input == "ERROR: NO IDENTITY TO TRANSFER":
					GuiPopupNewPasswordInputOKCancel(_(PI_NAME) + " - " + _("Create ssh identity"),
													 _("You don't have any ssh identity.") + "\n" +
													 _("If you want to create one, enter a passphrase for it and press OK:"),
													 "", (self.__create_identity__, None, GRAB_DEFAULT), (self.__create_identity_cancel__, None)).show()
				elif self.input == "ERROR: CONNECTION CLOSED":
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Connection closed"),
									 _("The connection was closed by %s@%s. Please ensure that sshd is running on the remote host.") % (self.user, self.host),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
				else:
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Unknown error"),
									 _("An unknown error '%s' has occurred.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
					self.__delete__()
			#WARNING[: .*]
			elif find(self.input, "WARNING") == 0:
				if self.input == "WARNING: IDENTITY ALLREADY UPLOADED":
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Ssh identity already installed"),
									 _("Your ssh identity has already been uploaded to %s@%s.") % (self.user, self.host),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
				else:
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Unknown warning"),
									 _("An unknown warning '%s' was issued.") % self.input,
									 (None, None), ((_("OK"), None, None, TRUE),)).show()
			elif self.input == "TRANSFERING IDENTITY":
				self.label.set_text(_("Transferring identity to %s@%s...") % (self.user, self.host))
			elif self.input == "INSTALLING IDENTITY":
				self.label.set_text(_("Installing identity on %s@%s...") % (self.user, self.host))
			elif self.input == "DONE":
				self.__delete__()
			self.input = ""
		else:
			self.input = self.input + input
	def __initiate_transfer__(self):
		self.transfer, self.transferin = popen2(PI_COMMANDS_DIR + "/unsetlang " + PI_COMMANDS_DIR + "/upload_identity.exp \"" + self.user + "\" \"" + self.host + "\" 2>&1", 0)
		self.transferin.write("PASSWORD: " + self.passwd + "\n")
		self.input_tag = _gtk.gtk_input_add(self.transfer.fileno(), INPUT_READ, self.__got_input__);
	def __password_ok__(self, text):
		self.passwd = text
		self.__initiate_transfer__()  
	def __password_cancel__(self, text):
		self.__delete__()
	def __create_identity_cancel__(self):
		self.__delete__()
	def __create_identity__(self, passwd):
		CreateIdentity(passwd, (self.__identity_created__, None), (self.__identity_creation_error__, None))
	def __identity_created__(self):
		self.__initiate_transfer__()
	def __identity_creation_error__(self):
		GuiPopupQuestion(_(PI_NAME) + " - " + _("Unable to create ssh identity"),
						 _("An error occurred while creating your ssh identity. This may be a bug, or a) you don't have permission to create files in your own homedir, or b) you don't have ssh properly installed."), (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
		self.__delete__()
	def __init__(self, passwd="", user="", host=""):
		self.passwd = passwd
		self.user = user
		self.host = host
		self.input = ""
		self.main = GtkWindow()
		self.main.set_title(_(PI_NAME) + " - " + _("Uploading ssh identity"))
		self.main.set_border_width(BORDER_WIDTH)
		self.label = GtkLabel(_("Connecting to %s@%s...") % (self.user, self.host))
		self.main.add(self.label)
		self.main.show_all()
		self.__initiate_transfer__()
	def __delete__(self):
		if not self.transfer.closed:
			_gtk.gtk_input_remove(self.input_tag)
			self.transfer.close()
		self.label.destroy()
		self.main.destroy()
		del self
	def ui_delete(self, *args):
		self.__delete__()
		return TRUE

class CreateIdentity:
	def __got_input__(self, *args):
		input = self.creation.read(1)
		if input == "":
			if not self.creation.closed:
				_gtk.gtk_input_remove(self.input_tag)
				self.creation.close()
			if self.cancel[0]:
				if self.cancel[1]:
					apply(self.cancel[0], self.cancel[1])
				else:
					apply(self.cancel[0], ())
			del self
		elif input == "\n":
			# These are the input categories:
			#WARNING[: .*]
			if find(self.input, "WARNING") == 0:
				if self.input == "WARNING: IDENTITY ALLREADY CREATED":
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Ssh identity already created"),
									 _("You already have an ssh identity."),
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
				else:
					GuiPopupQuestion(_(PI_NAME) + " - " + _("Unknown warning"),
									 _("An unknown warning '%s' was issued.") % self.input,
									 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
			elif self.input == "DONE":
				if not self.creation.closed:
					_gtk.gtk_input_remove(self.input_tag)
					self.creation.close()
				if self.cont[0]:
					if self.cont[1]:
						apply(self.cont[0], self.cont[1])
					else:
						apply(self.cont[0], ())
				del self
				return
			self.input = ""
		else:
			self.input = self.input + input
	def __init__(self, passwd="", cont=(None, None), cancel=(None, None)):
		self.cont = cont
		self.cancel = cancel
		self.passwd = passwd
		self.input = ""
		self.creation, self.creationin = popen2(PI_COMMANDS_DIR + "/unsetlang " + PI_COMMANDS_DIR + "/create_identity.exp 2>&1", 0)
		self.creationin.write("PASSWORD: " + self.passwd + "\n")
		self.input_tag = _gtk.gtk_input_add(self.creation.fileno(), INPUT_READ, self.__got_input__);

class ItemDescription(GtkListItem):
	icon_some_is_newer = None
	icon_none_is_newer = None
	def local_desc(self):
		return self.local
	def remote_desc(self):
		if self.protocol == CT_FTP:
			proto = "ftp"
		elif self.protocol == CT_RSH:
			proto = "rsh"
		else:
			proto = "ssh"
		return proto + "://" + self.user + "@" + self.host + ":" + self.remote
	def pretext(self):
		if self.newer == ST_REMOTE_IS_NEWER:
			return self.remote_desc()   
		else:
			return self.local_desc()
	def posttext(self):
		if self.newer == ST_REMOTE_IS_NEWER:
			return self.local_desc()
		else:
			return self.remote_desc()   
	def __init__(self, local, user, host, remote, newer, protocol):
		ItemDescription.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.local = local
		self.user = user
		self.host = host
		self.remote = remote
		self.newer = newer
		self.protocol = protocol
		self.set_border_width(BORDER_WIDTH)
		if not ItemDescription.icon_some_is_newer:
			ItemDescription.icon_some_is_newer = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/some_is_newer.xpm")
		if not ItemDescription.icon_none_is_newer:
			ItemDescription.icon_none_is_newer = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/none_is_newer.xpm")
		self.box = GtkHBox()
		self.prelabel = GtkLabel(self.pretext())
		self.box.pack_start(self.prelabel, FALSE, FALSE, 0)
		if self.newer == ST_EITHER_IS_NEWER:
			self.direction = GtkPixmap(self.icon_none_is_newer[0], self.icon_none_is_newer[1])
		else:
			self.direction = GtkPixmap(self.icon_some_is_newer[0], self.icon_some_is_newer[1])
		self.box.pack_start(self.direction, FALSE, FALSE, 0)
		self.postlabel = GtkLabel(self.posttext())
		self.box.pack_start(self.postlabel, FALSE, FALSE, 0)
		self.add(self.box)

	def show(self):
		self.prelabel.show()
		self.direction.show()
		self.postlabel.show()
		self.box.show()
		ItemDescription.__bases__[0].show(self)
	def set(self, local, user, host, remote, newer, protocol):
		self.local = local
		self.user = user
		self.host = host
		self.remote = remote
		self.newer = newer
		self.protocol = protocol
		self.prelabel.set_text(self.pretext())
		self.postlabel.set_text(self.posttext())
		if self.newer == ST_EITHER_IS_NEWER:
			self.direction.set(self.icon_none_is_newer[0], self.icon_none_is_newer[1])
		else:
			self.direction.set(self.icon_some_is_newer[0], self.icon_some_is_newer[1])

class Item(GtkWindow):
	def __init__(self, local="", user="", host="", remote="", newer=ST_EITHER_IS_NEWER, protocol=CT_SSH, delete_files=TRUE): 
		Item.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.connect("delete_event", self.ui_close)
		self.set_title(_(PI_NAME) + " - " + _("Edit connection"))
		self.set_border_width(BORDER_WIDTH)
		self.description = ItemDescription(local, user, host, remote, newer, protocol)
		_gtk.gtk_object_set_data(self.description._o, "item", self)
		self.box = GtkVBox(FALSE, BORDER_WIDTH)
		self.mainbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.leftbox = GtkVBox(FALSE, BORDER_WIDTH)
		###### Local #####
		self.localframe = GtkFrame(_("Local"))
		self.localbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.localbox.set_border_width(BORDER_WIDTH)
		self.localdirlabel = GtkLabel(_("Directory:"))
		self.localbox.pack_start(self.localdirlabel, FALSE, FALSE, 0)
		self.localdir = GtkEntry()
		self.localdir.set_text(local)
		self.localbox.pack_start(self.localdir, TRUE, TRUE, 0)
		self.localdirselector = GtkButton(_("..."))
		self.localdirselector.connect("clicked", self.ui_selectlocaldir)
		self.localbox.pack_start(self.localdirselector, FALSE, FALSE, 0)
		self.localframe.add(self.localbox)
		self.leftbox.pack_start(self.localframe, FALSE, FALSE, 0)
		##### Remote #####
		self.remoteframe = GtkFrame(_("Remote"))
		self.remotebox = GtkVBox(FALSE, BORDER_WIDTH)
		self.remotebox.set_border_width(BORDER_WIDTH)
		self.remoteuserbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.remoteuserlabel = GtkLabel(_("User:"))
		self.remoteuserbox.pack_start(self.remoteuserlabel, FALSE, FALSE, 0)
		self.remoteuser = GtkEntry()
		self.remoteuser.set_text(user)
		self.remoteuserbox.pack_start(self.remoteuser, TRUE, TRUE, 0)
		self.remotebox.pack_start(self.remoteuserbox, FALSE, FALSE, 0)
		self.remotehostbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.remotehostlabel = GtkLabel(_("Host:"))
		self.remotehostbox.pack_start(self.remotehostlabel, FALSE, FALSE, 0)
		self.remotehost = GtkEntry()
		self.remotehost.set_text(host)
		self.remotehostbox.pack_start(self.remotehost, TRUE, TRUE, 0)
		self.remotebox.pack_start(self.remotehostbox, FALSE, FALSE, 0)
		self.remotedirbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.remotedirlabel = GtkLabel(_("Directory:"))
		self.remotedirbox.pack_start(self.remotedirlabel, FALSE, FALSE, 0)
		self.remotedir = GtkEntry()
		self.remotedir.set_text(remote)
		self.remotedirbox.pack_start(self.remotedir, TRUE, TRUE, 0)
		self.remotebox.pack_start(self.remotedirbox, FALSE, FALSE, 0)
		self.remoteupload = GtkButton(_("Upload public ssh key to remote host"))
		self.remoteupload.connect("clicked", self.ui_upload)
		self.remotebox.pack_start(self.remoteupload, FALSE, FALSE, 0)
		self.remoteframe.add(self.remotebox)
		self.leftbox.pack_start(self.remoteframe, TRUE, TRUE, 0)
		self.mainbox.pack_start(self.leftbox, TRUE, TRUE, 0)
		##### Sync ######
		self.syncframe = GtkFrame(_("Synchronization"))
		self.syncbox = GtkVBox(FALSE, BORDER_WIDTH)
		##### Direction #####
		self.syncdirectionframe = GtkFrame(_("Direction:"))
		self.syncdirectionframe.set_shadow_type(SHADOW_NONE)
		self.syncdirectionbox = GtkVBox()
		self.syncdirection_none_is_newer = GtkRadioButton(label=_("Copy most recent files"))
		if newer == ST_EITHER_IS_NEWER:
			self.syncdirection_none_is_newer.set_active(TRUE)
		self.syncdirectionbox.pack_start(self.syncdirection_none_is_newer, FALSE, FALSE, 0)
		self.syncdirection_local_is_newer = GtkRadioButton(self.syncdirection_none_is_newer, _("Copy local files to remote"))
		if newer == ST_LOCAL_IS_NEWER:
			self.syncdirection_local_is_newer.set_active(TRUE)
		self.syncdirectionbox.pack_start(self.syncdirection_local_is_newer, FALSE, FALSE, 0)
		self.syncdirection_remote_is_newer = GtkRadioButton(self.syncdirection_local_is_newer, _("Copy remote files to local"))
		if newer == ST_REMOTE_IS_NEWER:
			self.syncdirection_remote_is_newer.set_active(TRUE)
		self.syncdirectionbox.pack_start(self.syncdirection_remote_is_newer, FALSE, FALSE, 0)
		self.syncdirectionframe.add(self.syncdirectionbox)
		self.syncbox.pack_start(self.syncdirectionframe, FALSE, FALSE, 0)
		##### Protocol #####
		self.syncprotocolframe = GtkFrame(_("Protocol:"))
		self.syncprotocolframe.set_shadow_type(SHADOW_NONE)
		self.syncprotocolbox = GtkVBox()
		self.syncprotocolbox_ssh = GtkRadioButton(label=_("Rsync over SSH (Secure SHell) - secure"))
		if protocol == CT_SSH:
			self.syncprotocolbox_ssh.set_active(TRUE)
		if not have_rsync or not have_ssh:
			self.syncprotocolbox_ssh.set_sensitive(FALSE)
		self.syncprotocolbox_ssh.connect("toggled", self.ui_protocol_ssh_toggle)
		self.ui_protocol_ssh_toggle(self.syncprotocolbox_ssh)
		self.syncprotocolbox.pack_start(self.syncprotocolbox_ssh, FALSE, FALSE, 0)
		self.syncprotocolbox_rsh = GtkRadioButton(self.syncprotocolbox_ssh, _("Rsync over RSH (Remote SHell) - insecure"))
		if protocol == CT_RSH:
			self.syncprotocolbox_rsh.set_active(TRUE)
		if not have_rsync or not have_rsh:
			self.syncprotocolbox_rsh.set_sensitive(FALSE)
		self.syncprotocolbox.pack_start(self.syncprotocolbox_rsh, FALSE, FALSE, 0)
		self.syncprotocolbox_ftp = GtkRadioButton(self.syncprotocolbox_rsh, _("FTP (File Transfer Protocol) - insecure"))
		if protocol == CT_FTP:
			self.syncprotocolbox_ftp.set_active(TRUE)
		self.syncprotocolbox.pack_start(self.syncprotocolbox_ftp, FALSE, FALSE, 0)
		self.syncprotocolframe.add(self.syncprotocolbox)
		self.syncbox.pack_start(self.syncprotocolframe, FALSE, FALSE, 0)
		##### Options ######
		self.syncoptionsframe = GtkFrame(_("Options:"))
		self.syncoptionsframe.set_shadow_type(SHADOW_NONE)
		self.syncoptionsbox = GtkVBox()
		self.syncoptions_delete = GtkCheckButton(_("Delete files"))
		self.syncoptions_delete.set_active(delete_files)
		self.syncdirection_none_is_newer.connect("toggled", self.ui_direction_toggle)
		self.ui_direction_toggle(self.syncdirection_none_is_newer)
		self.syncoptionsbox.pack_start(self.syncoptions_delete, FALSE, FALSE, 0)
		self.syncoptionsframe.add(self.syncoptionsbox)
		self.syncbox.pack_start(self.syncoptionsframe, FALSE, FALSE, 0)
		self.syncframe.add(self.syncbox)
		self.mainbox.pack_start(self.syncframe, FALSE, FALSE, 0)
		self.box.pack_start(self.mainbox, TRUE, TRUE, 0)
		##### Buttonbox/Actions ######
		self.buttonbox = GtkHBox(FALSE, BORDER_WIDTH)
		self.syncbtn = GtkButton(_("Synchronize"))
		self.syncbtn.connect("clicked", self.ui_sync)
		self.buttonbox.pack_start(self.syncbtn, FALSE, FALSE, 0)
		self.close = GtkButton(_("Close"))
		self.close.connect("clicked", self.ui_close)
		self.buttonbox.pack_start(self.close, FALSE, FALSE, 0)
		self.box.pack_start(self.buttonbox, FALSE, FALSE, 0)
		self.add(self.box)
	def newer(self):
		if self.syncdirection_local_is_newer.get_active():
			return ST_LOCAL_IS_NEWER
		elif self.syncdirection_remote_is_newer.get_active():
			return ST_REMOTE_IS_NEWER
		elif self.syncdirection_none_is_newer.get_active():
			return ST_EITHER_IS_NEWER
	def protocol(self):
		if self.syncprotocolbox_rsh.get_active():
			return CT_RSH
		elif self.syncprotocolbox_ftp.get_active():
			return CT_FTP
		elif self.syncprotocolbox_ssh.get_active():
			return CT_SSH
	def show(self):
		self.localdirlabel.show()
		self.localdir.show()
		self.localdir.grab_focus()
		self.localdirselector.show()
		self.localbox.show()
		self.localframe.show()
		self.remoteuserlabel.show()
		self.remoteuser.show()
		self.remoteuserbox.show()
		self.remotehostlabel.show()
		self.remotehost.show()
		self.remotehostbox.show()
		self.remotedirlabel.show()
		self.remotedir.show()
		self.remotedirbox.show()
		self.remoteupload.show()
		self.remotebox.show()
		self.remoteframe.show()
		self.leftbox.show()
		self.syncdirection_none_is_newer.show()
		self.syncdirection_local_is_newer.show()
		self.syncdirection_remote_is_newer.show()
		self.syncdirectionbox.show()
		self.syncdirectionframe.show()
		self.syncprotocolbox_ssh.show()
		self.syncprotocolbox_rsh.show()
		self.syncprotocolbox_ftp.show()
		self.syncprotocolbox.show()
		self.syncprotocolframe.show()
		self.syncoptions_delete.show()
		self.syncoptionsbox.show()
		self.syncoptionsframe.show()
		self.syncbox.show()
		self.syncframe.show()
		self.mainbox.show()
		self.close.show()
		self.syncbtn.show()
		self.buttonbox.show()
		self.box.show()
		Item.__bases__[0].show(self)
	def __repr__(self):
		return "Item(local = " + repr(self.localdir.get_text()) + ", user = " + repr(self.remoteuser.get_text()) + ", host = " + repr(self.remotehost.get_text()) + ", remote = " + repr(self.remotedir.get_text()) + ", newer = " + ST_STRINGS[self.newer()] + ", protocol = " + CT_STRINGS[self.protocol()] + ", delete_files = " + repr(self.syncoptions_delete.get_active()) + ")"
	def sync(self, *args):
		if self.protocol() == CT_FTP:
			FTPTransfer(self.newer(),
						self.syncoptions_delete.get_active(),
						self.remotehost.get_text(),
						self.remoteuser.get_text(),
						self.remotedir.get_text(),
						self.localdir.get_text())
		else:
			# Check that we can actually launch the synchronization (are the requested programs available ?)
			missing_program = None
			if not have_rsync:
				missing_program = "rsync"
			elif (self.protocol() == CT_SSH) and not have_ssh:
				missing_program = "ssh"
			elif (self.protocol() == CT_RSH) and not have_rsh:
				missing_program = "rsh"
			if missing_program:
				GuiPopupQuestion(_(PI_NAME) + " - " + _("External program not found"),
								 _("Could not find the required program '%s' for synchronization.\n"
								   "Please make sure it is correctly installed and that your PATH\n"
								   "contains the installation directory for it.") % missing_program,
								 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
			else:
				Transfer(self.protocol(),
						 self.localdir.get_text(),
						 self.remoteuser.get_text(),
						 self.remotehost.get_text(),
						 self.remotedir.get_text(),
						 self.newer(),
						 self.syncoptions_delete.get_active())
#Callbacks##Ui
	def ui_direction_toggle(self, button):
		self.syncoptions_delete.set_sensitive(not self.syncdirection_none_is_newer.get_active())
	def ui_protocol_ssh_toggle(self, button):
		self.remoteupload.set_sensitive(self.syncprotocolbox_ssh.get_active())
	def ui_upload(self, *args):
		GuiPopupPasswordInput(_(PI_NAME) + " - " + _("Passphrase required for host"),
							  _("Uploading ssh identity always requires a passphrase for the account.") + "\n" +
							  _("Please enter your passphrase for %s@%s:") % (self.remoteuser.get_text(), self.remotehost.get_text()),
							  "", (None, None), ((_("OK"), self.ui_upload_upload, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_upload_upload(self, passwd):
		TransferIdentity(passwd, self.remoteuser.get_text(), self.remotehost.get_text())
	def ui_close(self, *args):
		self.description.set(self.localdir.get_text(), self.remoteuser.get_text(), self.remotehost.get_text(), self.remotedir.get_text(), self.newer(), self.protocol())
		self.hide()
		return TRUE
	def ui_sync(self, *args):
		self.sync()
	def ui_selectlocaldir(self, *args):
		GuiPopupDirectorySelection(_(PI_NAME) + " - " + _("Please select local directory"),
								   self.localdir.get_text(), ok=(self.ui_selectlocaldirselected, None)).show()
	def ui_selectlocaldirselected(self, path):
		self.localdir.set_text(path)

class Profile(GtkList):
	def __init__(self, name, *items):
		Profile.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.connect("select-child", self.ui_select_child)
		self.connect("unselect-child", self.ui_unselect_child)
		self.label = GtkLabel(name)
		self.items = []
		self.current_item = None
		for item in items:
			self.do_add(item)
		
	def set_title(self, name):
		self.label.set_text(name)
		
	def do_add(self, item):
		# internal version without updates to the sensitive lists
		self.items.append(item)
		self.append_items([item.description])

	def add(self, item):
		self.do_add(item)
		update_sensitive_lists(gui.profiles)
		
	def do_remove(self, item):
		# internal version without updates to the sensitive lists
		if item == self.current():
			activate = self.child_position(item.description)
		else:
			activate = -1
		# We have to iterate backwards, to know that the indices are still
		# valid after a removal (So that we don't exceed the limit of the
		# array)
		for index in range(len(self.items) - 1, -1, -1):
			if self.items[index] == item:
				self.items[index:index+1] = []
		self.remove_items([item.description])
		if activate != -1:
			self.select_item(activate)

	def remove(self, item):
		self.do_remove(item)
		update_sensitive_lists(gui.profiles)
		
	def current(self):
		return self.current_item
	
	def show(self):
		for item in self.items:
			item.description.show()
		self.label.show()
		Profile.__bases__[0].show(self)
		
	def __repr__(self):
		res = "Profile(" + repr(self.label.get())
		for item in self.items:
			res = res + ", " + repr(item)
		res = res + ")"
		return res
	
	def sync(self):
		for item in self.items:
			item.sync()

#Callbacks##Ui
	def ui_select_child(self, list, child):
		self.current_item = _gtk.gtk_object_get_data(child._o, "item")
		# if we could select an item, then a group is bound to exist
		set_sensitive(sensitive_list_connection_if_active, TRUE)
		set_sensitive(sensitive_list_connection_if_inactive, TRUE)
		
	def ui_unselect_child(self, list, child):
		if self.current_item == _gtk.gtk_object_get_data(child._o, "item"):
			self.current_item = None
		# if we could select an item, then a group is bound to exist
		set_sensitive(sensitive_list_connection_if_active, FALSE)
		set_sensitive(sensitive_list_connection_if_inactive, TRUE)

class Profiles(GtkNotebook):
	def __init__(self, *profiles):
		Profiles.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.signal_connect_after("switch-page", self.ui_switch_page)
		self.set_scrollable(TRUE)
		self.set_tab_pos(POS_TOP)
		self.profiles = []
		for profile in profiles:
			self.do_add(profile)

		# Popup menu
		item_factory = GtkItemFactory(GtkMenu, "<main>", GtkAccelGroup())
		item_factory.create_items([
			( MNP_ALL_SYNC,				None,	self.mn_all_sync_handler,				0,		None			),
			( MNP_SEPARATOR,			None,	None,									0,		"<Separator>"	),
			( MNP_GROUP_SYNC,			None,	self.mn_group_sync_handler,				0,		None			),
			( MNP_GROUP_ADD,			None,	self.mn_group_add_handler,				0,		None			),
			( MNP_GROUP_REMOVE,			None,	self.mn_group_remove_handler,			0,		None			),
			( MNP_GROUP_RENAME,			None,	self.mn_group_rename_handler,			0,		None			),
			( MNP_SEPARATOR,			None,	None,									0,		"<Separator>"	),
			( MNP_CONNECTION_SYNC,		None,	self.mn_connection_sync_handler,		0,		None			),
			( MNP_CONNECTION_EDIT,		None,	self.mn_connection_edit_handler,		0,		None			),
			( MNP_CONNECTION_ADD,		None,	self.mn_connection_add_handler,			0,		None			),
			( MNP_CONNECTION_REMOVE,	None,	self.mn_connection_remove_handler,		0,		None			)
			])
		self.popup_menu = item_factory.get_widget("<main>")
		self.signal_connect_object("button_press_event", self.__popup_menu_handler, self.popup_menu)

		# Update sensitivity widget lists
		sensitive_list_sync_all.append(					item_factory.get_widget(strip_underscore( MNP_ALL_SYNC			)))
		sensitive_list_sync_group.append(				item_factory.get_widget(strip_underscore( MNP_GROUP_SYNC		)))
		sensitive_list_sync_connection.append(			item_factory.get_widget(strip_underscore( MNP_CONNECTION_SYNC	)))
		sensitive_list_group_if_inactive.append(		item_factory.get_widget(strip_underscore( MNP_GROUP_ADD			)))
		sensitive_list_group_if_active.extend([			item_factory.get_widget(strip_underscore( MNP_GROUP_SYNC		)),
														item_factory.get_widget(strip_underscore( MNP_GROUP_REMOVE		)),
														item_factory.get_widget(strip_underscore( MNP_GROUP_RENAME		))])
		sensitive_list_connection_if_inactive.append(	item_factory.get_widget(strip_underscore( MNP_CONNECTION_ADD	)))
		sensitive_list_connection_if_active.extend([	item_factory.get_widget(strip_underscore( MNP_CONNECTION_SYNC	)),
														item_factory.get_widget(strip_underscore( MNP_CONNECTION_EDIT	)),
														item_factory.get_widget(strip_underscore( MNP_CONNECTION_REMOVE	))])

# Callbacks: /All
	def mn_all_sync_handler(self, *args):
		if (self.get_current_page() != -1):
			self.sync()

# Callbacks: /Group
	def mn_group_sync_handler(self, *args):
		profile = self.current()
		if profile:
			profile.sync()

	def mn_group_do_add_handler(self, name):
		profile = Profile(name)
		profile.show()
		self.add(profile)
		
	def mn_group_add_handler(self, *args):
		GuiPopupInput(_(PI_NAME) + " - " + _("New group"),
					  _("Please enter a name for the new group:"), "", (None, None),
					  ((_("OK"), self.mn_group_do_add_handler, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()

	def mn_group_do_remove_handler(self, profile, *args):
		self.remove(profile)
	
	def mn_group_remove_handler(self, *args):
		profile = self.current()
		if profile:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove group"),
							 _("Remove the current group and all its connections?"), (None, None),
							 ((_("Yes"), self.mn_group_do_remove_handler, (profile,)), (_("No"), None, None, GRAB_DEFAULT))).show()

	def mn_group_do_rename_handler(self, name, profile):
		profile.set_title(name)
	
	def mn_group_rename_handler(self, *args):
		profile = self.current()
		if profile:
			GuiPopupInput(_(PI_NAME) + " - " + _("Rename group"),
						  _("Please enter the new name for the group:"), "", (None, None),
						  ((_("OK"), self.mn_group_do_rename_handler, (profile,), GRAB_DEFAULT), (_("Cancel"), None, None))).show()

# Callbacks: /Connection
	def mn_connection_sync_handler(self, *args):
		profile = self.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.sync()
	
	def mn_connection_edit_handler(self, *args):
		profile = self.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.show()
	
	def mn_connection_add_handler(self, *args):
		profile = self.current()
		if profile:
			item = Item()
			item.description.show()
			item.show()
			profile.add(item)

	def mn_connection_do_remove_handler(self, profile, connection):
		profile.remove(connection)
	
	def mn_connection_remove_handler(self, *args):
		profile = self.current()
		if profile:
			connection = profile.current()
			if connection:
				GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove connection"),
								 _("Remove the current connection?"), (None, None),
								 ((_("Yes"), self.mn_connection_do_remove_handler, (profile, connection)), (_("No"), None, None, GRAB_DEFAULT))).show()

# Callbacks: popup menu
	def __popup_menu_handler(self, menu, event):
		if isinstance(menu, GtkMenu) and (event.type == BUTTON_PRESS) and (event.button == 3):
			self.popup_menu.popup(None, None, None, event.button, event.time)
			retval = TRUE
		else:
			retval = FALSE
		return retval

	def do_add(self, profile):
		# internal version without updates to the sensitive lists
		self.profiles.append(profile)
		scroller = GtkScrolledWindow()
		scroller.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		scroller.add_with_viewport(profile)
		scroller.show()
		_gtk.gtk_object_set_data(scroller._o, "profile", profile)
		_gtk.gtk_object_set_data(profile._o, "scroller", scroller)
		self.append_page(scroller, profile.label)

	def add(self, profile):
		self.do_add(profile)
		update_sensitive_lists(self)
		
	def do_remove(self, profile):
		# internal version without updates to the sensitive lists
		pagenum = self.page_num(_gtk.gtk_object_get_data(profile._o, "scroller"))
		if pagenum != -1:
			self.remove_page(pagenum)
		for index in range(len(self.profiles) - 1, -1, -1):
			if self.profiles[index] == profile:
				self.profiles[index:index+1] = []

	def remove(self, profile):
		self.do_remove(profile)
		update_sensitive_lists(self)
		
	def current(self):
		page = self.get_current_page()
		if page != -1:
			return _gtk.gtk_object_get_data(self.get_nth_page(page)._o, "profile")
		else:
			return None

	def ui_switch_page(self, *args):
		update_sensitive_lists(self)
		
	def show(self):
		for profile in self.profiles:
			profile.show()
		Profiles.__bases__[0].show(self)
		
	def __repr__(self):
		res = "Profiles("
		sep = ""
		for profile in self.profiles:
			res = res + sep + repr(profile)
			sep = ", "
		res = res + ")"
		return res
	
	def sync(self):
		for profile in self.profiles:
			profile.sync()

class GuiMenubar(GtkHandleBox):

	def __init__(self, gui):
		GuiMenubar.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.gui = gui
		accel_group = GtkAccelGroup()
		item_factory = GtkItemFactory(GtkMenuBar, "<main>", accel_group)
		item_factory.create_items([
			( MN_FILE,					None,			None,					0,		"<Branch>"		),
			( MN_FILE_SYNC_ALL,			"<control>Y",	self.ui_file_sync,		0,		None			),
			( MN_FILE_SAVE_AND_QUIT,	"<control>S",	self.ui_file_quit,		0,		None			),
			( MN_FILE_QUIT_WO_SAVING,	"<control>Q",	self.ui_file_exit,		0,		None			),
			( MN_GROUP,					None,			None,					0,		"<Branch>"		),
			( MN_GROUP_SYNC,			None,			self.ui_group_sync,		0,		None			),
			( MN_GROUP_ADD,				None,			self.ui_group_add,		0,		None			),
			( MN_GROUP_REMOVE,			None,			self.ui_group_remove,	0,		None			),
			( MN_GROUP_RENAME,			None,			self.ui_group_rename,	0,		None			),
			( MN_CONNECTION,			None,			None,					0,		"<Branch>"		),
			( MN_CONNECTION_SYNC,		None,			self.ui_conn_sync,		0,		None			),
			( MN_CONNECTION_ADD,		None,			self.ui_conn_add,		0,		None			),
			( MN_CONNECTION_REMOVE,		None,			self.ui_conn_remove,	0,		None			),
			( MN_CONNECTION_EDIT,		None,			self.ui_conn_edit,		0,		None			),
			( MN_HELP,					None,			None,					0,		"<Branch>"		),
			( MN_HELP_MANUAL,			None,			self.ui_help_help,		0,		None			),
			( MN_HELP_SEPARATOR,		None,			None,					0,		"<Separator>"	),
			( MN_HELP_MANDRAKE_CAMPUS,	None,			self.ui_help_campus,	0,		None			),
			( MN_HELP_MANDRAKE_EXPERT,	None,			self.ui_help_expert,	0,		None			),
			( MN_HELP_SEPARATOR,		None,			None,					0,		"<Separator>"	),
			( MN_HELP_BUG_REPORT,		None,			self.ui_help_bugreport,	0,		None			),
			( MN_HELP_SEPARATOR,		None,			None,					0,		"<Separator>"	),
			( MN_HELP_ABOUT,			None,			self.ui_help_about,		0,		None			)
			])
		self.gui.add_accel_group(accel_group)
		self.menubar = item_factory.get_widget("<main>")
		self.add(self.menubar)

		# Update sensitivity widget lists
		sensitive_list_sync_all.append(					item_factory.get_widget(strip_underscore( MN_FILE_SYNC_ALL		)))
		sensitive_list_sync_group.append(				item_factory.get_widget(strip_underscore( MN_GROUP_SYNC			)))
		sensitive_list_sync_connection.append(			item_factory.get_widget(strip_underscore( MN_CONNECTION_SYNC	)))
		sensitive_list_group_if_inactive.append(		item_factory.get_widget(strip_underscore( MN_GROUP_ADD			)))
		sensitive_list_group_if_active.extend([			item_factory.get_widget(strip_underscore( MN_GROUP_SYNC			)),
														item_factory.get_widget(strip_underscore( MN_GROUP_REMOVE		)),
														item_factory.get_widget(strip_underscore( MN_GROUP_RENAME		))])
		sensitive_list_connection_if_inactive.append(	item_factory.get_widget(strip_underscore( MN_CONNECTION_ADD		)))
		sensitive_list_connection_if_active.extend([	item_factory.get_widget(strip_underscore( MN_CONNECTION_SYNC	)),
														item_factory.get_widget(strip_underscore( MN_CONNECTION_EDIT	)),
														item_factory.get_widget(strip_underscore( MN_CONNECTION_REMOVE	))])
	
	def show(self):
		self.menubar.show()
		GuiMenubar.__bases__[0].show(self)

# Callbacks: /File
	def ui_file_sync(self, *args):
		self.gui.profiles.sync()
	def ui_file_quit(self, *args):
		self.gui.ui_close()
	def ui_file_exit(self, *args):
		GuiPopupQuestion(_(PI_NAME) + " - " + _("Quit without saving"),
						 _("Quit without saving changes?"), (None, None),
						 ((_("Yes"), self.ui_file_do_exit, None), (_("No"), None, None, GRAB_DEFAULT))).show()
	def ui_file_do_exit(self):
		mainquit()

# Callbacks: /Group
	def ui_group_add(self, *args):
		GuiPopupInput(_(PI_NAME) + " - " + _("New group"),
					  _("Please enter a name for the new group:"), "", (None, None),
					  ((_("OK"), self.ui_group_do_add, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_group_do_add(self, name):
		profile = Profile(name)
		profile.show()
		self.gui.profiles.add(profile)
	def ui_group_remove(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove group"),
							 _("Remove the current group and all its connections?"), (None, None),
							 ((_("Yes"), self.ui_group_do_remove, (self.gui.profiles, profile)), (_("No"), None, None, GRAB_DEFAULT))).show()
	def ui_group_do_remove(self, profiles, profile):
		profiles.remove(profile)
	def ui_group_rename(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			GuiPopupInput(_(PI_NAME) + " - " + _("Rename group"),
						  _("Please enter the new name for the group:"), "", (None, None),
						  ((_("OK"), self.ui_group_do_rename, (profile,), GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_group_do_rename(self, name, profile):
		profile.set_title(name)
	def ui_group_sync(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			profile.sync()

# Callbacks: /Connection
	def ui_add_group_with_connection_add(self, name):
		profile = Profile(name)
		profile.show()
		self.gui.profiles.add(profile)
		self.ui_conn_add()
	def ui_conn_add(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			item = Item()
			item.description.show()
			item.show()
			profile.add(item)
		else:
			GuiPopupInput(_(PI_NAME) + " - " + _("New group"),
						  _("Please enter a name for the new group:"), "", (None, None),
						  ((_("OK"), self.ui_add_group_with_connection_add, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_conn_remove(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove connection"),
								 _("Remove the current connection?"), (None, None),
								 ((_("Yes"), self.ui_conn_do_remove, (profile, connection)), (_("No"), None, None, GRAB_DEFAULT))).show()
	def ui_conn_do_remove(self, profile, connection):
		profile.remove(connection)
	def ui_conn_edit(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.show()
	def ui_conn_sync(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.sync()

# Callbacks: /Help
	def ui_help_about(self, *args):
		GuiPopupQuestion(_(PI_NAME) + " - " + _("About"), PI_ABOUT, (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()
	def ui_help_help(self, *args):
		show_help()
	def browse_web(self, url):
		if os.environ.has_key("BROWSER"):
			command = ("/bin/sh", ("-c", os.environ["BROWSER"] + " " + url,))
		else:
			command = (None, ())
		apply(spawn, command)
	def ui_help_campus(self, *args):
		self.browse_web(URL_MANDRAKE_CAMPUS)
	def ui_help_expert(self, *args):
		self.browse_web(URL_MANDRAKE_EXPERT)
	def ui_help_bugreport(self, *args):
		GuiPopupQuestion(_(PI_NAME) + " - " + _("Bug report"),
						 _("If you wish to report a bug, please connect to\n%s\nand fill in a bug report") % URL_MANDRAKE_QA,
						 (None, None), ((_("OK"), None, None, GRAB_DEFAULT),)).show()

class GuiToolbarTool(GtkToolbar):
	def __init__(self, gui):
		GuiToolbarTool.__bases__[0].__init__(self, ORIENTATION_HORIZONTAL, TOOLBAR_BOTH)
		self.set_button_relief(RELIEF_NONE)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.gui = gui
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/add_group.xpm")
		button = self.append_item(_("Add\nGroup"), _("Add a new group of connections"), "", GtkPixmap(icon[0], icon[1]), self.ui_add_group)
		sensitive_list_group_if_inactive.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/add_connection.xpm")
		button = self.append_item(_("Add\nConnection"), _("Add a new connection"), "", GtkPixmap(icon[0], icon[1]), self.ui_add_connection)
		sensitive_list_connection_if_inactive.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/remove_group.xpm")
		button = self.append_item(_("Remove\nGroup"), _("Remove the current group of connections"), "", GtkPixmap(icon[0], icon[1]), self.ui_remove_group)
		sensitive_list_group_if_active.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/remove_connection.xpm")
		button = self.append_item(_("Remove\nConnection"), _("Remove the current connection"), "", GtkPixmap(icon[0], icon[1]), self.ui_remove_connection)
		sensitive_list_connection_if_active.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/edit_connection.xpm")
		button = self.append_item(_("Edit\nConnection"), _("Edit the current connection"), "", GtkPixmap(icon[0], icon[1]), self.ui_edit_connection)
		sensitive_list_connection_if_active.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/sync_all.xpm")
		button = self.append_item(_("Synchronize\nAll"), _("Synchronize all connections (in all groups)"), "", GtkPixmap(icon[0], icon[1]), self.ui_sync_all)
		sensitive_list_sync_all.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/sync_group.xpm")
		button = self.append_item(_("Sync\nGroup"), _("Synchronize all connections in the current group"), "", GtkPixmap(icon[0], icon[1]), self.ui_sync_group)
		sensitive_list_sync_group.append(button)
		sensitive_list_group_if_active.append(button)
		icon = create_pixmap_from_xpm(self, self.get_style().bg[STATE_NORMAL], PI_ICONS_DIR + "/sync_connection.xpm")
		button = self.append_item(_("Sync\nConnection"), _("Synchronize the current connection"), "", GtkPixmap(icon[0], icon[1]), self.ui_sync_connection)
		sensitive_list_sync_connection.append(button)
		sensitive_list_connection_if_active.append(button)
		
#Callbacks
	def ui_add_group(self, *args):
		GuiPopupInput(_(PI_NAME) + " - " + _("New group"),
					  _("Please enter a name for the new group:"), "", (None, None),
					  ((_("OK"), self.ui_add_group_add, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_add_group_add(self, name):
		profile = Profile(name)
		profile.show()
		self.gui.profiles.add(profile)
	def ui_add_group_with_connection_add(self, name):
		profile = Profile(name)
		profile.show()
		self.gui.profiles.add(profile)
		self.ui_add_connection()
	def ui_add_connection(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			item = Item()
			item.description.show()
			item.show()
			profile.add(item)
		else:
			GuiPopupInput(_(PI_NAME) + " - " + _("New group"),
						  _("Please enter a name for the new group:"), "", (None, None),
						  ((_("OK"), self.ui_add_group_with_connection_add, None, GRAB_DEFAULT), (_("Cancel"), None, None))).show()
	def ui_remove_group(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove group"),
							 _("Remove the current group and all its connections?"), (None, None),
							 ((_("Yes"), self.ui_remove_group_remove, (self.gui.profiles, profile)), (_("No"), None, None, GRAB_DEFAULT))).show()
	def ui_remove_group_remove(self, profiles, profile):
		profiles.remove(profile)
	def ui_remove_connection(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				GuiPopupQuestion(_(PI_NAME) + " - " + _("Remove connection"),
								 _("Remove the current connection?"), (None, None),
								 ((_("Yes"), self.ui_remove_connection_remove, (profile, connection)), (_("No"), None, None, GRAB_DEFAULT))).show()
	def ui_remove_connection_remove(self, profile, connection):
		profile.remove(connection)
	def ui_sync_all(self, *args):
		self.gui.profiles.sync()
	def ui_sync_group(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			profile.sync()
	def ui_sync_connection(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.sync()
	def ui_edit_connection(self, *args):
		profile = self.gui.profiles.current()
		if profile:
			connection = profile.current()
			if connection:
				connection.show()

class GuiToolbar(GtkHandleBox):
	def __init__(self, gui):
		GuiToolbar.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.toolbar = GuiToolbarTool(gui)
		self.add(self.toolbar)  
	def show(self):
		self.toolbar.show()
		GuiToolbar.__bases__[0].show(self)

class Gui(GtkWindow):
	def __init__(self, profiles):
		Gui.__bases__[0].__init__(self)
		_gtk.gtk_object_set_data(self._o, "userclass", self)
		self.set_default_size(512, 384)
		self.set_title(_(PI_NAME))
		self.connect("delete_event", self.ui_close)
		self.box = GtkVBox()
		if not profiles:
			profiles = Profiles()
		self.menubar = GuiMenubar(self)
		self.box.pack_start(self.menubar, FALSE, FALSE, 0)
		self.toolbar = GuiToolbar(self)
		self.box.pack_start(self.toolbar, FALSE, FALSE, 0)
		self.profiles = profiles
		self.box.pack_start(self.profiles, TRUE, TRUE, 0)
		self.add(self.box)
	def show(self):
		self.menubar.show()
		self.toolbar.show()
		self.profiles.show()
		self.box.show()
		Gui.__bases__[0].show(self)
	def __repr__(self):
		return "draksync[\"profiles\"] = " + repr(self.profiles)
#Callbacks##Ui
	def ui_close(self, *args):
		save()
		self.hide()
		mainquit()

draksync = {"profiles":None}
load()
gui = Gui(draksync["profiles"])
update_sensitive_lists(gui.profiles)
gui.show()
mainloop()
